define class WSSearch as Session
	cErrorMessage      = ''
	cProcessingMessage = ''
	dimension aResults[1, 3]

* Open the WSSearch table when the object is created.

	function Init
		use WSSEARCH order ORDER
	endfunc

	function Search(tcSearch, tdBegin, tdEnd)
		local lnResults, ;
			lcName, ;
			lcCode, ;
			lcUserName, ;
			lcPassword, ;
			loWS, ;
			loSearch, ;
			llOK, ;
			lcMessage, ;
			lcXML
		with This

* Ensure we have valid parameters.

			if vartype(tcSearch) <> 'C' or empty(tcSearch)
				.cErrorMessage = 'Invalid search string'
				return -1
			endif vartype(tcSearch) <> 'C' ...
			if vartype(tdBegin) <> 'D' or empty(tdBegin)
				.cErrorMessage = 'Invalid starting date'
				return -1
			endif vartype(tdBegin) <> 'D' ...
			if vartype(tdEnd) <> 'D' or empty(tdEnd)
				.cErrorMessage = 'Invalid starting date'
				return -1
			endif vartype(tdEnd) <> 'D' ...

* Do a search against all active services.

			lnResults = 0
			scan for ACTIVE

* Add a new row to the array and set the processing message.

				.cErrorMessage = ''
				lcName         = alltrim(NAME)
				lcCode         = SEARCH
				lcUserName     = alltrim(USERNAME)
				lcPassword     = alltrim(PASSWORD)
				lnResults      = lnResults + 1
				dimension .aResults[lnResults, 3]
				.aResults[lnResults, 1] = lcName
				.cProcessingMessage     = 'Accessing ' + lcName

* Instantiate our subclass of the FFC WSClient class and set it up.

				loWS         = createobject('WSClient3')
				loWS.cWSName = lcName
				loSearch     = loWS.SetupClient(alltrim(WSDL), '', '')
				lcMessage    = loWS.cErrorMessage
				llOK         = vartype(loSearch) = 'O' and empty(lcMessage)

* If we succeeded in creating the Web Service proxy, perform the search by
* executing the code in the SEARCH memo.

				if llOK
					.cProcessingMessage = 'Searching ' + lcName
					lcXML = execscript(lcCode, loSearch, tcSearch, ;
						tdBegin, tdEnd, lcUserName, lcPassword)
*!*	lcFile = sys(2015)
*!*	copy memo SEARCH to (lcFile + '.prg')
*!*	lcXML = evaluate(lcFile + '(loSearch, tcSearch, tdBegin, tdEnd, lcUserName, lcPassword)')
*!*	erase (lcFile + '.prg')
*!*	erase (lcFile + '.fxp')
					.aResults[lnResults, 2] = not empty(lcXML) and ;
						'<?xml' $ lcXML
					do case
						case not empty(lcXML)
							.aResults[lnResults, 3] = lcXML
						case not empty(.cErrorMessage)
							.aResults[lnResults, 3] = .cErrorMessage
						otherwise
							.aResults[lnResults, 3] = 'No results'
					endcase

* We couldn't create the Web Service object, so set the error message.

				else
					.aResults[lnResults, 3] = lcMessage
				endif llOK
			endscan for ACTIVE
			.cProcessingMessage = 'Done'
			.cErrorMessage      = ''
		endwith
		return lnResults
	endfunc

* Handle any errors by either calling the ON ERROR handler or just setting
* our cErrorMessage property.

	function Error(tnError, tcMethod, tnLine)
		local lcError
		lcError = upper(on('ERROR'))
		if empty(lcError)
			This.cErrorMessage = message()
		else
			lcError = strtran(lcError, 'SYS(16)',   '"' + tcMethod + '"')
			lcError = strtran(lcError, 'PROGRAM()', '"' + tcMethod + '"')
			lcError = strtran(lcError, ',ERROR()',  ',tnError')
			lcError = strtran(lcError, ' ERROR()',  ' tnError')
			lcError = strtran(lcError, 'LINENO()',  'tnLine')
			&lcError
		endif empty(lcError)
	endfunc

* Close the WSSearch table when the object is destroyed.

	function Destroy
		use in (select('WSSEARCH'))
	endfunc
enddefine

* Subclass of WSClient2 in FFC _WebServices.vcx that works a little better.

define class WSClient3 as WSClient2 of (home() + 'ffc\_webservices.vcx')
	cErrorMessage = ''

	function ErrorAlert(tcMessage)
		This.cErrorMessage = tcMessage
	endfunc

	function SetupClient(tcURI, tcService, tcPort)
		local lnSelect, ;
			luReturn
		lnSelect = select()
		luReturn = dodefault(tcURI, tcService, tcPort)
		select (lnSelect)
		return luReturn
	endfunc

	function CheckWSDBF
		local llReturn
		if not empty(This.cWSAlias) and used(This.cWSAlias)
			select (This.cWSAlias)
			llReturn = .T.
		else
			llReturn = dodefault()
		endif not empty(This.cWSAlias) ...
		return llReturn
	endfunc
enddefine
